@@ -1,6 +1,8 @@
 /*
-   ѥȥꥷںץ for 
-*/
+ * make patricia tree for ChaSen
+ *
+ * $Id: interface.c,v 1.7 2001/03/02 00:09:17 masayu-a Exp $
+ */
 
 #include <stdio.h>
 #include <string.h>
@@ -12,20 +14,21 @@
 static pat_t *dicfile[PAT_DIC_NUM];
 static int ndicfile;
 
-static char line[50000]; /* Ϲ */
-static char inkey[10000]; /*  */
+static char line[50000];	/* input line */
+static char inkey[10000];	/* key (search/insert) */
 
 static int command(void);
 static void lookup(char *key);
 static void make_pat_from_file(char *textname);
-static void insert_from_file(pat_t *pat);
+static void insert_from_file(pat_t * pat);
 static char *seek_eol(register char *p, const char *eof);
-static void prog_bar(FILE *fp, int count);
+static void prog_bar(FILE * fp, int count);
 
 /*
  * Usage
  */
-static void usage(void)
+static void
+usage(void)
 {
     printf("Usage: pattool [-F filename]\n");
     printf("\tNo option    ---  Interpreter Mode\n");
@@ -36,9 +39,10 @@ static void usage(void)
 /*
  * Main
  */
-int main(int argc, char *argv[])
+int
+main(int argc, char *argv[])
 {
-    if (argc == 1) /* no option */
+    if (argc == 1)		/* no option */
 	command();
     else if (argc >= 3 && (strcmp(argv[1], "-F") == 0)) {
 	char textname[CHA_FILENAME_LEN];
@@ -48,10 +52,12 @@ int main(int argc, char *argv[])
 	basename = argv[2];
 	printf("Make Index File \"%s.pat\" from \"%s.int\"\n",
 	       basename, basename);
-	/* ե륪ץ  ѥڤŤ*/
+
+	/* file open & make patricia tree */
 	sprintf(textname, "%s.int", basename);
 	make_pat_from_file(textname);
-	/* ѥڥ */
+
+	/* save patricia tree */
 	sprintf(patname, "%s.pat", basename);
 	pat_save(dicfile[0], patname);
     } else
@@ -63,7 +69,8 @@ int main(int argc, char *argv[])
 /*
  * Command interpriter
  */
-static int command(void)
+static int
+command(void)
 {
     char comm;
     int i;
@@ -72,26 +79,27 @@ static int command(void)
     char textname[CHA_FILENAME_LEN];
     char patname[CHA_FILENAME_LEN];
 
-    ndicfile = 0; /* Ѥ뼭(ѥ)ο */
+    ndicfile = 0;		/* the number of dictionary */
 
     printf("Interpreter Mode:  \'q\' exit, '?\' help\n");
 
-    /* ޥɥ󥿥ץ꥿ */
+    /* Command interpriter */
     while (1) {
-	printf("> "); fflush(stdout);
+	printf("> ");
+	fflush(stdout);
 
-	fgets(line, sizeof(line), stdin); /* ޥɤ */
+	fgets(line, sizeof(line), stdin); /* input of command */
 
-	sscanf(line,"%c %s", &comm, inkey); 
-	switch(comm) {
+	sscanf(line, "%c %s", &comm, inkey);
+	switch (comm) {
 
-	case 'F': /* ե뤫 */
+	case 'F':		/* insert from file*/
 	    sprintf(textname, "%s.int", inkey);
 	    make_pat_from_file(textname);
 	    break;
 
-	case 'e': /* θ(exact match) */
-	    if (ndicfile == 0) { /* 顼 */
+	case 'e':		/* search key (exact match) */
+	    if (ndicfile == 0) {	/* process ERROR */
 		fprintf(stderr,
 			"!!! make or load PAT data before SEARCH.\n");
 		break;
@@ -103,8 +111,8 @@ static int command(void)
 	    printf("%s", rslt[0]);
 	    break;
 
-	case 'j': /* ʸμ(ChaSenʳ) */
-	    if (ndicfile == 0){ /* 顼 */
+	case 'j': /* lookup dictionary for sentence (1 step of ChaSen) */
+	    if (ndicfile == 0) {	/* process ERROR */
 		fprintf(stderr,
 			"!!! make or load PAT data before SEARCH.\n");
 		break;
@@ -112,31 +120,26 @@ static int command(void)
 	    lookup(inkey);
 	    break;
 
-	case 'O': /* ڤν */
-	    inkey[0] = '\0';
-	    pat_show_patfile(dicfile[0], stdout, inkey); /* ɸ */
-	    break;
-
-	case 'S': /* ڤΥ */
+	case 'S':		/* save tree */
 	    strcat(inkey, ".pat");
 	    pat_save(dicfile[0], inkey);
 	    break;
 
-	case 'L': /* ڤΥ */
+	case 'L':		/* load tree */
 	    strcpy(strtmp, inkey);
 	    sprintf(textname, "%s.int", inkey);
 	    sprintf(patname, "%s.pat", inkey);
-	    /* ѥΥץ */
+	    /* open corpus */
 	    dicfile[ndicfile] = pat_open(textname, patname);
 	    ndicfile++;
 	    break;
 
-	case 'q': /* 󥿥ץ꥿νλ */
-	case 'Q': /* 󥿥ץ꥿νλ */
+	case 'q':		/* end of interpreter */
+	case 'Q':	        /* end of interpreter */
 	    printf("QUIT\n");
 	    return 0;
 
-	default: /* إ */
+	default:		/* help */
 	    printf("command:\n\
               \tj [string] --- ChaSen search\n\
               \te [string] --- exact match search\n\
@@ -147,16 +150,18 @@ static int command(void)
 	    break;
 	}
 
-	printf("ok\n"); fflush(stdout);  /* ץץ */
+	printf("ok\n");
+	fflush(stdout);		/* prompt */
     }
 }
 
 
-static void make_pat_from_file(char *textname)
+static void
+make_pat_from_file(char *textname)
 {
     if ((dicfile[ndicfile] = pat_open(textname, NULL))
 	== NULL) {
-	fprintf(stderr,"No such File: %s\n", textname);
+	fprintf(stderr, "No such File: %s\n", textname);
 	exit(1);
     }
 
@@ -164,7 +169,8 @@ static void make_pat_from_file(char *textname)
     ndicfile++;
 }
 
-static void insert_from_file(pat_t *pat)
+static void
+insert_from_file(pat_t * pat)
 {
     int count = 0;
     char *c, *bof, *eof;
@@ -181,18 +187,20 @@ static void insert_from_file(pat_t *pat)
     printf("read %d words, %d bytes\n", count, c - bof);
 }
 
-static char *seek_eol(register char *p, const char *eof)
+static char *
+seek_eol(register char *p, const char *eof)
 {
     for (; p < eof; p++) {
-        if (*p == '\n')
-            return p + 1;
+	if (*p == '\n')
+	    return p + 1;
     }
     return p;
 }
 
-static void prog_bar(FILE *fp, int count)
+static void
+prog_bar(FILE * fp, int count)
 {
-    if (count % 10000 == 0){
+    if (count % 10000 == 0) {
 	fputc('.', fp);
 	if (count % 200000 == 0)
 	    fprintf(fp, " %d\n", count);
@@ -200,7 +208,8 @@ static void prog_bar(FILE *fp, int count)
     }
 }
 
-static void lookup(char *key)
+static void
+lookup(char *key)
 {
     int dic_no, i;
     char *dic_buffer[256], *dic_bof;
@@ -212,4 +221,3 @@ static void lookup(char *key)
 	    printf("%s: %d\n", dic_buffer[i], dic_buffer[i] - dic_bof);
     }
 }
-
